@@ -1,16 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2015 -- leonerd@leonerd.org.uk
 
 package Future::Utils;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.23';
+our $VERSION = '0.33';
 
 use Exporter 'import';
+# Can't import the one from Exporter as it relies on package inheritance
+sub export_to_level
+{
+   my $pkg = shift; local $Exporter::ExportLevel = 1 + shift; $pkg->import(@_);
+}
 
 our @EXPORT_OK = qw(
    call
@@ -28,6 +33,8 @@ our @EXPORT_OK = qw(
 use Carp;
 our @CARP_NOT = qw( Future );
 
+use Future;
+
 =head1 NAME
 
 C<Future::Utils> - utility functions for working with C<Future> objects
@@ -101,11 +108,18 @@ Z<>
     return $item_f;
  } foreach => \@items, concurrent => 10;
 
+Unless otherwise noted, the following functions require at least version
+I<0.08>.
+
 =cut
 
 =head1 INVOKING A BLOCK OF CODE
 
-=head2 $f = call { CODE }
+=head2 call
+
+   $f = call { CODE }
+
+I<Since version 0.22.>
 
 The C<call> function invokes a block of code that returns a future, and simply
 returns the future it returned. The code is wrapped in an C<eval {}> block, so
@@ -124,7 +138,11 @@ sub call(&)
    return Future->call( $code );
 }
 
-=head2 $f = call_with_escape { CODE }
+=head2 call_with_escape
+
+   $f = call_with_escape { CODE }
+
+I<Since version 0.22.>
 
 The C<call_with_escape> function invokes a block of code that returns a
 future, and passes in a separate future (called here an "escape future").
@@ -174,7 +192,9 @@ If some specific subclass or instance of C<Future> is required as the return
 value, it can be passed as the C<return> argument. Otherwise the return value
 will be constructed by cloning the first non-immediate trial C<Future>.
 
-=head2 $future = repeat { CODE } while => CODE
+=head2 repeat+while
+
+   $future = repeat { CODE } while => CODE
 
 Repeatedly calls the C<CODE> block while the C<while> condition returns a true
 value. Each time the trial future completes, the C<while> condition is passed
@@ -187,7 +207,9 @@ If the C<$code> block dies entirely and throws an exception, this will be
 caught and considered as an immediately-failed C<Future> with the exception as
 the future's failure. The exception will not be propagated to the caller.
 
-=head2 $future = repeat { CODE } until => CODE
+=head2 repeat+until
+
+   $future = repeat { CODE } until => CODE
 
 Repeatedly calls the C<CODE> block until the C<until> condition returns a true
 value. Each time the trial future completes, the C<until> condition is passed
@@ -196,13 +218,17 @@ the trial future.
  $trial_f = $code->( $previous_trial_f )
  $accept = $until->( $trial_f )
 
-=head2 $future = repeat { CODE } foreach => ARRAY, otherwise => CODE
+=head2 repeat+foreach
+
+   $future = repeat { CODE } foreach => ARRAY, otherwise => CODE
+
+I<Since version 0.13.>
 
 Calls the C<CODE> block once for each value obtained from the array, passing
 in the value as the first argument (before the previous trial future). When
 there are no more items left in the array, the C<otherwise> code is invoked
-once and passed the last trial future, if there was one, otherwise C<undef> if
-the list was originally empty. The result of the eventual future will be the
+once and passed the last trial future, if there was one, or C<undef> if the
+list was originally empty. The result of the eventual future will be the
 result of the future returned from C<otherwise>.
 
 The referenced array may be modified by this operation.
@@ -211,11 +237,21 @@ The referenced array may be modified by this operation.
  $final_f = $otherwise->( $last_trial_f )
 
 The C<otherwise> code is optional; if not supplied then the result of the
-eventual future will simply be that of the last trial.
+eventual future will simply be that of the last trial. If there was no trial,
+because the C<foreach> list was already empty, then an immediate successful
+future with an empty result is returned.
 
-=head2 $future = repeat { CODE } foreach => ARRAY, while => CODE, ...
+=head2 repeat+foreach+while
 
-=head2 $future = repeat { CODE } foreach => ARRAY, until => CODE, ...
+   $future = repeat { CODE } foreach => ARRAY, while => CODE, ...
+
+I<Since version 0.13.>
+
+=head2 repeat+foreach+until
+
+   $future = repeat { CODE } foreach => ARRAY, until => CODE, ...
+
+I<Since version 0.13.>
 
 Combines the effects of C<foreach> with C<while> or C<until>. Calls the
 C<CODE> block once for each value obtained from the array, until the array is
@@ -227,7 +263,11 @@ operation is terminated early due to the C<while> or C<until> condition being
 satisfied, the eventual result will simply be that of the last trial that was
 executed.
 
-=head2 $future = repeat { CODE } generate => CODE, otherwise => CODE
+=head2 repeat+generate
+
+   $future = repeat { CODE } generate => CODE, otherwise => CODE
+
+I<Since version 0.13.>
 
 Calls the C<CODE> block once for each value obtained from the generator code,
 passing in the value as the first argument (before the previous trial future).
@@ -245,11 +285,17 @@ The generator is called in list context but should return only one item per
 call. Subsequent values will be ignored. When it has no more items to return
 it should return an empty list.
 
+For backward compatibility this function will allow a C<while> or C<until>
+condition that requests a failure be repeated, but it will print a warning if
+it has to do that. To apply repeating behaviour that can catch and retry
+failures, use C<try_repeat> instead. This old behaviour is now deprecated and
+will be removed in the next version.
+
 =cut
 
 sub _repeat
 {
-   my ( $code, $return, $trialp, $cond, $sense ) = @_;
+   my ( $code, $return, $trialp, $cond, $sense, $is_try ) = @_;
 
    my $prev = $$trialp;
 
@@ -262,12 +308,19 @@ sub _repeat
          $return ||= $trial->new;
          $trial->on_ready( sub {
             return if $$trialp->is_cancelled;
-            _repeat( $code, $return, $trialp, $cond, $sense );
+            _repeat( $code, $return, $trialp, $cond, $sense, $is_try );
          });
          return $return;
       }
 
-      if( !$cond->( $trial ) ^ $sense ) {
+      my $stop;
+      if( not eval { $stop = !$cond->( $trial ) ^ $sense; 1 } ) {
+         $return ||= $trial->new;
+         $return->fail( $@ );
+         return $return;
+      }
+
+      if( $stop ) {
          # Return result
          $return ||= $trial->new;
          $trial->on_done( $return );
@@ -275,6 +328,10 @@ sub _repeat
          return $return;
       }
 
+      if( !$is_try and $trial->failure ) {
+         carp "Using Future::Utils::repeat to retry a failure is deprecated; use try_repeat instead";
+      }
+
       # redo
       undef $$trialp;
    }
@@ -322,7 +379,7 @@ sub repeat(&@)
             goto &$otherwise;
          }
          else {
-            return $last_trial_f;
+            return $last_trial_f || Future->done;
          }
       };
 
@@ -344,28 +401,42 @@ sub repeat(&@)
    my $future = $args{return};
 
    my $trial;
-   $args{while} and $future = _repeat( $code, $future, \$trial, $args{while}, 0 );
-   $args{until} and $future = _repeat( $code, $future, \$trial, $args{until}, 1 );
+   $args{while} and $future = _repeat( $code, $future, \$trial, $args{while}, 0, $args{try} );
+   $args{until} and $future = _repeat( $code, $future, \$trial, $args{until}, 1, $args{try} );
 
    $future->on_cancel( sub { $trial->cancel } );
 
    return $future;
 }
 
-=head2 $future = try_repeat { CODE } ...
+=head2 try_repeat
 
-Currently a simple alias to C<repeat>. However, in some later version the
-C<repeat> function will be changed so that if a trial future fails, then the
-eventual future will immediately fail as well, making its semantics a little
-closer to that of a C<while {}> loop in Perl. Code that specifically wishes
-to catch failures in trial futures and retry the block should use
-C<try_repeat> specifically.
+   $future = try_repeat { CODE } ...
+
+I<Since version 0.18.>
+
+A variant of C<repeat> that doesn't warn when the trial fails and the
+condition code asks for it to be repeated.
+
+In some later version the C<repeat> function will be changed so that if a
+trial future fails, then the eventual future will immediately fail as well,
+making its semantics a little closer to that of a C<while {}> loop in Perl.
+Code that specifically wishes to catch failures in trial futures and retry
+the block should use C<try_repeat> specifically.
 
 =cut
 
-*try_repeat = \&repeat;
+sub try_repeat(&@)
+{
+   # defeat prototype
+   &repeat( @_, try => 1 );
+}
 
-=head2 $future = try_repeat_until_success { CODE } ...
+=head2 try_repeat_until_success
+
+   $future = try_repeat_until_success { CODE } ...
+
+I<Since version 0.18.>
 
 A shortcut to calling C<try_repeat> with an ending condition that simply tests
 for a successful result from a future. May be combined with C<foreach> or
@@ -385,7 +456,8 @@ sub try_repeat_until_success(&@)
    defined($args{while}) or defined($args{until})
       and croak "Cannot pass 'while' or 'until' to try_repeat_until_success";
 
-   try_repeat( $code, while => sub { shift->failure }, %args );
+   # defeat prototype
+   &try_repeat( $code, while => sub { shift->failure }, %args );
 }
 
 # Legacy name
@@ -456,6 +528,9 @@ The expected return value from each item's future, and the value returned from
 the result future will differ in each function's case; they are documented
 below.
 
+For similarity with perl's core C<map> function, the item is also available
+aliased as C<$_>.
+
 =cut
 
 # This function is invoked in two circumstances:
@@ -485,7 +560,7 @@ sub _fmap_slot
             return $return;
          }
 
-         my $f = $slots->[$idx] = Future->call( $code, $item );
+         my $f = $slots->[$idx] = Future->call( $code, local $_ = $item );
 
          if( $collect eq "array" ) {
             push @$results, my $r = [];
@@ -570,7 +645,11 @@ sub _fmap
    return $future;
 }
 
-=head2 $future = fmap_concat { CODE } ...
+=head2 fmap_concat
+
+   $future = fmap_concat { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> expects each item future to return a list of zero or
 more values, and the overall result will be the concatenation of all these
@@ -594,12 +673,16 @@ sub fmap_concat(&@)
    my %args = @_;
 
    _fmap( $code, %args, collect => "array" )->then( sub {
-      return Future->new->done( map { @$_ } @_ );
+      return Future->done( map { @$_ } @_ );
    });
 }
 *fmap = \&fmap_concat;
 
-=head2 $future = fmap_scalar { CODE } ...
+=head2 fmap_scalar
+
+   $future = fmap_scalar { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> acts more like the C<map> functions found in Scheme or
 Haskell; it expects that each item future returns only one value, and the
@@ -621,7 +704,11 @@ sub fmap_scalar(&@)
 }
 *fmap1 = \&fmap_scalar;
 
-=head2 $future = fmap_void { CODE } ...
+=head2 fmap_void
+
+   $future = fmap_void { CODE } ...
+
+I<Since version 0.14.>
 
 This version of C<fmap> does not collect any results from its item futures, it
 simply waits for them all to complete. Its result future will provide no
