@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
 
 package Test::Future;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Test::Builder::Module );
 
-our $VERSION = '0.32';
+our $VERSION = '0.33';
 
 our @EXPORT = qw(
    no_pending_futures
@@ -45,7 +45,11 @@ code based on, or using L<Future> instances or subclasses.
 
 =cut
 
-=head2 no_pending_futures( \&code, $name )
+=head2 no_pending_futures
+
+   no_pending_futures( \&code, $name )
+
+I<Since version 0.29.>
 
 Runs the given block of code, while keeping track of every C<Future> instance
 constructed while doing so. After the code has returned, each of these
